import { CallOptions, RpcResponse, SubscriptionResponse, RpcStringResult, RpcPrefixedHexStringResult, RpcValidTypeResult, RpcBooleanResult, RpcSyncingResult, RpcAccountsResult, RpcBlockResult, RpcTransactionResult, RpcTransactionReceiptResult, RpcStringArrayResult, RpcCompiledSolidityResult, RpcLogResult } from 'web3-providers-base/lib/types';
import { PrefixedHexString, EthTransaction, BlockIdentifier, EthFilter, ValidTypes } from 'web3-utils/lib/types';
import { Web3EthOptions, EthCallTransaction } from './types';
export default class Web3Eth {
    private _requestManager;
    private _defaultReturnType;
    constructor(options: Web3EthOptions);
    private _send;
    private _subscribe;
    private static _isBlockTag;
    /**
     * Returns the current client version
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Client version
     */
    getClientVersion(callOptions?: CallOptions): Promise<RpcStringResult | SubscriptionResponse>;
    /**
     * Returns Keccak-256 (not the standardized SHA3-256) of the given data
     * @param {string} data Data to convert into SHA3 hash
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} SHA3 hash of {data}
     */
    getSha3(data: string, callOptions?: CallOptions): Promise<RpcPrefixedHexStringResult | SubscriptionResponse>;
    /**
     * Returns the current network version
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Current network version
     */
    getNetworkVersion(callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns true if client is actively listening for network connections
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} true if currently listening, otherwise false
     */
    getNetworkListening(callOptions?: CallOptions): Promise<RpcBooleanResult | SubscriptionResponse>;
    /**
     * Returns number of peers currently connected to the client
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} true if currently listening, otherwise false
     */
    getNetworkPeerCount(callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns the current ethereum protocol version
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} The current ethereum protocol version
     */
    getProtocolVersion(callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns an object with data about the sync status or false when not syncing
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Object with sync status data or false when not syncing
     */
    getSyncing(callOptions?: CallOptions): Promise<RpcSyncingResult | SubscriptionResponse>;
    /**
     * Returns the client's coinbase address
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} The current coinbase address
     */
    getCoinbase(callOptions?: CallOptions): Promise<RpcPrefixedHexStringResult | SubscriptionResponse>;
    /**
     * Returns true if client is actively mining new blocks
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} true if the client is mining, otherwise false
     */
    getMining(callOptions?: CallOptions): Promise<RpcBooleanResult | SubscriptionResponse>;
    /**
     * Returns the number of hashes per second that the node is mining with
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Number of hashes per second
     */
    getHashRate(callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns the current price per gas in wei
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing current gas price in wei
     */
    getGasPrice(callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns a list of addresses owned by client.
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Array of addresses owned by the client
     */
    getAccounts(callOptions?: CallOptions): Promise<RpcAccountsResult | SubscriptionResponse>;
    /**
     * Returns the number of most recent block
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing current block number client is on
     */
    getBlockNumber(callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns the balance of the account of given address
     * @param {string} address Address to get balance of
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing current balance in wei
     */
    getBalance(address: PrefixedHexString, blockIdentifier: BlockIdentifier, callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns the value from a storage position at a given address
     * @param {string} address Address of storage to query
     * @param {string} storagePosition Hex string representing position in storage to retrieve
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing value at {storagePosition}
     */
    getStorageAt(address: PrefixedHexString, storagePosition: ValidTypes, blockIdentifier: BlockIdentifier, callOptions?: CallOptions): Promise<RpcPrefixedHexStringResult | SubscriptionResponse>;
    /**
     * Returns the number of transactions sent from an address
     * @param {string} address Address to get transaction count of
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing number of transactions sent by {address}
     */
    getTransactionCount(address: PrefixedHexString, blockIdentifier: BlockIdentifier, callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns the number of transactions in a block from a block matching the given block hash
     * @param {string} blockHash Hash of block to query transaction count of
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing number of transactions in block
     */
    getBlockTransactionCountByHash(blockHash: PrefixedHexString, callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns the number of transactions in a block from a block matching the given block number
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing number of transactions in block
     */
    getBlockTransactionCountByNumber(blockIdentifier: BlockIdentifier, callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns the number of uncles in a block from a block matching the given block hash
     * @param {string} blockHash Hash of block to query
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing number of uncles in block
     */
    getUncleCountByBlockHash(blockHash: PrefixedHexString, callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns the number of uncles in a block from a block matching the given block number
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing number of uncles in block
     */
    getUncleCountByBlockNumber(blockIdentifier: BlockIdentifier, callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns code at a given address
     * @param {string} address Address to get code at
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing the code at {address}
     */
    getCode(address: PrefixedHexString, blockIdentifier: BlockIdentifier, callOptions?: CallOptions): Promise<RpcPrefixedHexStringResult | SubscriptionResponse>;
    /**
     * Calculates an Ethereum specific signature
     * @param {string} address Address to use to sign {data}
     * @param {string} message Message to sign
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing signed message
     */
    sign(address: PrefixedHexString, message: PrefixedHexString, callOptions?: CallOptions): Promise<RpcPrefixedHexStringResult | SubscriptionResponse>;
    /**
     * Signs a transaction that can be submitted to the network at a later time using with sendRawTransaction
     * @param {object} transaction Ethereum transaction
     * @param {string} transaction.from Address transaction will be sent from
     * @param {string} transaction.to Address transaction is directed towards (optional when creating new contract)
     * @param {string} transaction.gas Hex string representing gas to provide for transaction (ETH node defaults to 90,000)
     * @param {string} transaction.gasPrice Hex string representing price paid for each unit of gas in Wei (ETH node will determine if not provided)
     * @param {string} transaction.value Hex string representing number of Wei to send to {to}
     * @param {string} transaction.data Hex string representing compiled code of a contract or the hash of the invoked method signature and encoded parameters
     * @param {string} transaction.nonce Can be used to overwrite pending transactions that use the same nonce
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing signed message
     */
    signTransaction(transaction: EthTransaction, callOptions?: CallOptions): Promise<RpcPrefixedHexStringResult | SubscriptionResponse>;
    /**
     * Submits a transaction object to the provider to be sign and sent to the network
     * @param {object} transaction Ethereum transaction
     * @param {string} transaction.from Address transaction will be sent from
     * @param {string} transaction.to Address transaction is directed towards (optional when creating new contract)
     * @param {string} transaction.gas Hex string representing gas to provide for transaction (ETH node defaults to 90,000)
     * @param {string} transaction.gasPrice Hex string representing price paid for each unit of gas in Wei (ETH node will determine if not provided)
     * @param {string} transaction.value Hex string representing number of Wei to send to {to}
     * @param {string} transaction.data Hex string representing compiled code of a contract or the hash of the invoked method signature and encoded parameters
     * @param {string} transaction.nonce Can be used to overwrite pending transactions that use the same nonce
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Transaction hash or zero hash if the transaction is not yet available
     */
    sendTransaction(transaction: EthTransaction, callOptions?: CallOptions): Promise<RpcPrefixedHexStringResult | SubscriptionResponse>;
    /**
     * Submits a previously signed transaction object to the network
     * @param {string} rawTransaction Hex string representing previously signed transaction
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Transaction hash or zero hash if the transaction is not yet available
     */
    sendRawTransaction(rawTransaction: PrefixedHexString, callOptions?: CallOptions): Promise<RpcPrefixedHexStringResult | SubscriptionResponse>;
    /**
     * Executes a new message call immediately without creating a transaction on the block chain
     * @param {object} transaction Ethereum transaction
     * @param {string} transaction.from Address transaction will be sent from
     * @param {string} transaction.to Address transaction is directed towards
     * @param {string} transaction.gas Hex string representing gas to provide for transaction (ETH node defaults to 90,000)
     * @param {string} transaction.gasPrice Hex string representing price paid for each unit of gas in Wei (ETH node will determine if not provided)
     * @param {string} transaction.value Hex string representing number of Wei to send to {to}
     * @param {string} transaction.data Hash of the method signature and encoded parameters
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing return value of executed contract
     */
    call(transaction: EthCallTransaction, callOptions?: CallOptions): Promise<RpcResponse | SubscriptionResponse>;
    /**
     * Generates and returns an estimate of how much gas is necessary to allow the transaction to complete
     * @param {object} transaction Ethereum transaction
     * @param {string} transaction.from Address transaction will be sent from (optional)
     * @param {string} transaction.to Address transaction is directed towards (optional)
     * @param {string} transaction.gas Hex string representing gas to provide for transaction (ETH node defaults to 90,000)
     * @param {string} transaction.gasPrice Hex string representing price paid for each unit of gas in Wei (ETH node will determine if not provided)
     * @param {string} transaction.value Hex string representing number of Wei to send to {to} (optional)
     * @param {string} transaction.data Hash of the method signature and encoded parameters (optional)
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing estimated amount of gas to be used
     */
    estimateGas(transaction: EthTransaction, callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Returns information about a block by hash
     * @param {string} blockHash Hash of block to get information for
     * @param {boolean} returnFullTxs If true it returns the full transaction objects, if false returns only the hashes of the transactions
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A block object or null when no block was found
     */
    getBlockByHash(blockHash: PrefixedHexString, returnFullTxs: boolean, callOptions?: CallOptions): Promise<RpcBlockResult | SubscriptionResponse>;
    /**
     * Returns information about a block by number
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {boolean} returnFullTxs If true it returns the full transaction objects, if false returns only the hashes of the transactions
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A block object or null when no block was found
     */
    getBlockByNumber(blockIdentifier: BlockIdentifier, returnFullTxs: boolean, callOptions?: CallOptions): Promise<RpcBlockResult | SubscriptionResponse>;
    /**
     * Returns the information about a transaction requested by transaction hash
     * @param {string} txHash Hash of transaction to retrieve
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A transaction object or {null} when no transaction was found
     */
    getTransactionByHash(txHash: PrefixedHexString, callOptions?: CallOptions): Promise<RpcTransactionResult | SubscriptionResponse>;
    /**
     * Returns information about a transaction by block hash and transaction index position
     * @param {string} blockHash Hash of block to get transactions of
     * @param {string} transactionIndex Hex string representing index of transaction to return
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A transaction object or {null} when no transaction was found
     */
    getTransactionByBlockHashAndIndex(blockHash: PrefixedHexString, transactionIndex: ValidTypes, callOptions?: CallOptions): Promise<RpcTransactionResult | SubscriptionResponse>;
    /**
     * Returns information about a transaction by block number and transaction index position
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {string} transactionIndex Hex string representing index of transaction to return
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A transaction object or {null} when no transaction was found
     */
    getTransactionByBlockNumberAndIndex(blockIdentifier: BlockIdentifier, transactionIndex: ValidTypes, callOptions?: CallOptions): Promise<RpcTransactionResult | SubscriptionResponse>;
    /**
     * Returns the receipt of a transaction by transaction hash
     * @param {string} txHash Hash of transaction to get receipt of
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A transaction object or {null} when no receipt was found
     */
    getTransactionReceipt(txHash: PrefixedHexString, callOptions?: CallOptions): Promise<RpcTransactionReceiptResult | SubscriptionResponse>;
    /**
     * Returns information about a uncle of a block by hash and uncle index position
     * @param {string} blockHash Hash of block to get uncles of
     * @param {string} uncleIndex Index of uncle to retrieve
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A block object or null when no block was found
     */
    getUncleByBlockHashAndIndex(blockHash: PrefixedHexString, uncleIndex: ValidTypes, callOptions?: CallOptions): Promise<RpcBlockResult | SubscriptionResponse>;
    /**
     * Returns information about a uncle of a block by number and uncle index position
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {string} uncleIndex Index of uncle to retrieve
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A block object or null when no block was found
     */
    getUncleByBlockNumberAndIndex(blockIdentifier: BlockIdentifier, uncleIndex: ValidTypes, callOptions?: CallOptions): Promise<RpcBlockResult | SubscriptionResponse>;
    /**
     * Returns a list of available compilers in the client
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A list of available compilers
     */
    getCompilers(callOptions?: CallOptions): Promise<RpcStringArrayResult | SubscriptionResponse>;
    /**
     * Returns compiled solidity code
     * @param {string} sourceCode Solidity code to be compiled
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} compiled {sourceCode}
     */
    compileSolidity(sourceCode: PrefixedHexString, callOptions?: CallOptions): Promise<RpcCompiledSolidityResult | SubscriptionResponse>;
    /**
     * Returns compiled LLL code
     * @param {string} sourceCode LLL code to be compiled
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} compiled {sourceCode}
     */
    compileLLL(sourceCode: PrefixedHexString, callOptions?: CallOptions): Promise<RpcPrefixedHexStringResult | SubscriptionResponse>;
    /**
     * Returns compiled serpent code
     * @param {string} sourceCode Serpent code to be compiled
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} compiled {sourceCode}
     */
    compileSerpent(sourceCode: PrefixedHexString, callOptions?: CallOptions): Promise<RpcPrefixedHexStringResult | SubscriptionResponse>;
    /**
     * Creates a filter object, based on filter options, to notify when the state changes (logs)
     * @param {object} filter Filter to be created
     * @param {string|number} filter.fromBlock Start filter at integer block number or "latest", "earliest", "pending"
     * @param {string|number} filter.toBlock End filter at integer block number or "latest", "earliest", "pending"
     * @param {string|string[]} filter.address: Contract address or list of addresses from which logs should originate
     * @param {string[]} filter.topics Topics to use for filtering (optional)
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Filter id
     */
    newFilter(filter: EthFilter, callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Creates a filter in the node, to notify when a new block arrives
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Filter id
     */
    newBlockFilter(callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Creates a filter in the node, to notify when new pending transactions arrive
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Filter id
     */
    newPendingTransactionFilter(callOptions?: CallOptions): Promise<RpcValidTypeResult | SubscriptionResponse>;
    /**
     * Uninstalls a filter with given id. Should always be called when watch is no longer needed
     * @param {string} filterId Id of filter to uninstall from node
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Returns true if filter was successfully uninstalled, otherwise false
     */
    uninstallFilter(filterId: ValidTypes, callOptions?: CallOptions): Promise<RpcBooleanResult | SubscriptionResponse>;
    /**
     * Polling method for a filter, which returns an array of logs which occurred since last poll
     * @param {string} filterid Id of filter to retrieve changes from
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Array of log objects, or an empty array if nothing has changed since last poll
     */
    getFilterChanges(filterId: ValidTypes, callOptions?: CallOptions): Promise<RpcLogResult | SubscriptionResponse>;
    /**
     * Returns an array of all logs matching filter with given id
     * @param {string} filterid Id of filter to retrieve
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Array of log objects, or an empty array if nothing has changed since last poll
     */
    getFilterLogs(filterId: ValidTypes, callOptions?: CallOptions): Promise<RpcLogResult | SubscriptionResponse>;
    /**
     * Returns an array of all logs matching a given filter object
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Array of log objects, or an empty array if nothing has changed since last poll
     */
    getLogs(filter: EthFilter, callOptions?: CallOptions): Promise<RpcLogResult | SubscriptionResponse>;
    /**
     * Returns the hash of the current block, the seedHash, and the boundary condition to be met (“target”)
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Array of work info (in order: current block header pow-hash, seed hash used for the DAG, and boundary condition (“target”), 2^256 / difficulty)
     */
    getWork(callOptions?: CallOptions): Promise<RpcStringArrayResult | SubscriptionResponse>;
    /**
     * Used for submitting a proof-of-work solution
     * @param {string} nonce Hex string representing found nonce (64 bits)
     * @param {string} powHash Hex string representing POW hash (256 bits)
     * @param {string} digest Hex string representing mix digest (256 bits)
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Returns true if the provided solution is valid, otherwise false
     */
    submitWork(nonce: ValidTypes, powHash: PrefixedHexString, digest: PrefixedHexString, callOptions?: CallOptions): Promise<RpcBooleanResult | SubscriptionResponse>;
    /**
     * Used for submitting mining hashrate
     * @param {string} hashRate Hex string representing desired hash rate (32 bytes)
     * @param {string} clientId Hex string representing ID identifying the client
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Returns true if the provided solution is valid, otherwise false
     */
    submitHashRate(hashRate: ValidTypes, clientId: ValidTypes, callOptions?: CallOptions): Promise<RpcBooleanResult | SubscriptionResponse>;
}
//# sourceMappingURL=index.d.ts.map