"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web3_core_requestmanager_1 = __importDefault(require("web3-core-requestmanager"));
const web3_utils_1 = require("web3-utils");
const types_1 = require("web3-utils/lib/types");
class Web3Eth {
    constructor(options) {
        this._requestManager = new web3_core_requestmanager_1.default({
            providerUrl: options.providerUrl,
        });
        this._defaultReturnType =
            options.returnType || types_1.ValidTypesEnum.PrefixedHexString;
    }
    _send(method, params, callOptions) {
        return this._requestManager.send({
            ...callOptions?.rpcOptions,
            method,
            params,
        }, callOptions?.providerCallOptions);
    }
    _subscribe(method, params, callOptions) {
        return this._requestManager.subscribe({
            ...callOptions?.rpcOptions,
            method,
            params,
        }, callOptions?.providerCallOptions);
    }
    static _isBlockTag(value) {
        return Object.values(types_1.BlockTags).includes(value);
    }
    /**
     * Returns the current client version
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Client version
     */
    async getClientVersion(callOptions) {
        try {
            const requestParameters = [
                'web3_clientVersion',
                [],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting client version: ${error.message}`);
        }
    }
    /**
     * Returns Keccak-256 (not the standardized SHA3-256) of the given data
     * @param {string} data Data to convert into SHA3 hash
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} SHA3 hash of {data}
     */
    async getSha3(data, callOptions) {
        try {
            const requestParameters = ['web3_sha3', [data], callOptions];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting sha3 hash: ${error.message}`);
        }
    }
    /**
     * Returns the current network version
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Current network version
     */
    async getNetworkVersion(callOptions) {
        try {
            const requestParameters = [
                'net_version',
                [],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting network version: ${error.message}`);
        }
    }
    /**
     * Returns true if client is actively listening for network connections
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} true if currently listening, otherwise false
     */
    async getNetworkListening(callOptions) {
        try {
            const requestParameters = [
                'net_listening',
                [],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting network listening: ${error.message}`);
        }
    }
    /**
     * Returns number of peers currently connected to the client
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} true if currently listening, otherwise false
     */
    async getNetworkPeerCount(callOptions) {
        try {
            const requestParameters = [
                'net_peerCount',
                [],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting network peer count: ${error.message}`);
        }
    }
    /**
     * Returns the current ethereum protocol version
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} The current ethereum protocol version
     */
    async getProtocolVersion(callOptions) {
        try {
            const requestParameters = [
                'eth_protocolVersion',
                [],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting protocol version: ${error.message}`);
        }
    }
    /**
     * Returns an object with data about the sync status or false when not syncing
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Object with sync status data or false when not syncing
     */
    async getSyncing(callOptions) {
        try {
            const requestParameters = [
                'eth_syncing',
                [],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: typeof response.result === 'boolean'
                    ? response.result
                    : web3_utils_1.formatOutputObject(response.result, ['startingBlock', 'currentBlock', 'highestBlock'], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting syncing status: ${error.message}`);
        }
    }
    /**
     * Returns the client's coinbase address
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} The current coinbase address
     */
    async getCoinbase(callOptions) {
        try {
            const requestParameters = [
                'eth_coinbase',
                [],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting coinbase address: ${error.message}`);
        }
    }
    /**
     * Returns true if client is actively mining new blocks
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} true if the client is mining, otherwise false
     */
    async getMining(callOptions) {
        try {
            const requestParameters = [
                'eth_mining',
                [],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting mining status: ${error.message}`);
        }
    }
    /**
     * Returns the number of hashes per second that the node is mining with
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Number of hashes per second
     */
    async getHashRate(callOptions) {
        try {
            const requestParameters = [
                'eth_hashrate',
                [],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting hash rate: ${error.message}`);
        }
    }
    /**
     * Returns the current price per gas in wei
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing current gas price in wei
     */
    async getGasPrice(callOptions) {
        try {
            const requestParameters = [
                'eth_gasPrice',
                [],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting gas price: ${error.message}`);
        }
    }
    /**
     * Returns a list of addresses owned by client.
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Array of addresses owned by the client
     */
    async getAccounts(callOptions) {
        try {
            const requestParameters = [
                'eth_accounts',
                [],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting accounts: ${error.message}`);
        }
    }
    /**
     * Returns the number of most recent block
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing current block number client is on
     */
    async getBlockNumber(callOptions) {
        try {
            const requestParameters = [
                'eth_blockNumber',
                [],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting block number: ${error.message}`);
        }
    }
    /**
     * Returns the balance of the account of given address
     * @param {string} address Address to get balance of
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing current balance in wei
     */
    async getBalance(address, blockIdentifier, callOptions) {
        try {
            const requestParameters = [
                'eth_getBalance',
                [
                    address,
                    Web3Eth._isBlockTag(blockIdentifier)
                        ? blockIdentifier
                        : web3_utils_1.toHex(blockIdentifier),
                ],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting balance: ${error.message}`);
        }
    }
    /**
     * Returns the value from a storage position at a given address
     * @param {string} address Address of storage to query
     * @param {string} storagePosition Hex string representing position in storage to retrieve
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing value at {storagePosition}
     */
    async getStorageAt(address, storagePosition, blockIdentifier, callOptions) {
        try {
            const requestParameters = [
                'eth_getStorageAt',
                [address, web3_utils_1.toHex(storagePosition), web3_utils_1.toHex(blockIdentifier)],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting storage value: ${error.message}`);
        }
    }
    /**
     * Returns the number of transactions sent from an address
     * @param {string} address Address to get transaction count of
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing number of transactions sent by {address}
     */
    async getTransactionCount(address, blockIdentifier, callOptions) {
        try {
            const requestParameters = [
                'eth_getTransactionCount',
                [address, web3_utils_1.toHex(blockIdentifier)],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting transaction count: ${error.message}`);
        }
    }
    /**
     * Returns the number of transactions in a block from a block matching the given block hash
     * @param {string} blockHash Hash of block to query transaction count of
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing number of transactions in block
     */
    async getBlockTransactionCountByHash(blockHash, callOptions) {
        try {
            const requestParameters = [
                'eth_getBlockTransactionCountByHash',
                [blockHash],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting transaction count for block by hash: ${error.message}`);
        }
    }
    /**
     * Returns the number of transactions in a block from a block matching the given block number
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing number of transactions in block
     */
    async getBlockTransactionCountByNumber(blockIdentifier, callOptions) {
        try {
            const requestParameters = [
                'eth_getBlockTransactionCountByNumber',
                [web3_utils_1.toHex(blockIdentifier)],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting transaction count for block by number: ${error.message}`);
        }
    }
    /**
     * Returns the number of uncles in a block from a block matching the given block hash
     * @param {string} blockHash Hash of block to query
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing number of uncles in block
     */
    async getUncleCountByBlockHash(blockHash, callOptions) {
        try {
            const requestParameters = ['eth_getUncleCountByBlockHash', [blockHash], callOptions];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting uncle count for block by hash: ${error.message}`);
        }
    }
    /**
     * Returns the number of uncles in a block from a block matching the given block number
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing number of uncles in block
     */
    async getUncleCountByBlockNumber(blockIdentifier, callOptions) {
        try {
            const requestParameters = [
                'eth_getUncleCountByBlockNumber',
                [web3_utils_1.toHex(blockIdentifier)],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting uncle count for block by number: ${error.message}`);
        }
    }
    /**
     * Returns code at a given address
     * @param {string} address Address to get code at
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing the code at {address}
     */
    async getCode(address, blockIdentifier, callOptions) {
        try {
            const requestParameters = ['eth_getCode', [address, web3_utils_1.toHex(blockIdentifier)], callOptions];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting code at address: ${error.message}`);
        }
    }
    /**
     * Calculates an Ethereum specific signature
     * @param {string} address Address to use to sign {data}
     * @param {string} message Message to sign
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing signed message
     */
    async sign(address, message, callOptions) {
        try {
            const requestParameters = ['eth_sign', [address, message], callOptions];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error signing message: ${error.message}`);
        }
    }
    /**
     * Signs a transaction that can be submitted to the network at a later time using with sendRawTransaction
     * @param {object} transaction Ethereum transaction
     * @param {string} transaction.from Address transaction will be sent from
     * @param {string} transaction.to Address transaction is directed towards (optional when creating new contract)
     * @param {string} transaction.gas Hex string representing gas to provide for transaction (ETH node defaults to 90,000)
     * @param {string} transaction.gasPrice Hex string representing price paid for each unit of gas in Wei (ETH node will determine if not provided)
     * @param {string} transaction.value Hex string representing number of Wei to send to {to}
     * @param {string} transaction.data Hex string representing compiled code of a contract or the hash of the invoked method signature and encoded parameters
     * @param {string} transaction.nonce Can be used to overwrite pending transactions that use the same nonce
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing signed message
     */
    async signTransaction(transaction, callOptions) {
        try {
            const requestParameters = [
                'eth_signTransaction',
                [
                    {
                        ...transaction,
                        gas: transaction.gas
                            ? web3_utils_1.toHex(transaction.gas)
                            : undefined,
                        gasPrice: transaction.gasPrice
                            ? web3_utils_1.toHex(transaction.gasPrice)
                            : undefined,
                        value: transaction.value
                            ? web3_utils_1.toHex(transaction.value)
                            : undefined,
                        nonce: transaction.nonce
                            ? web3_utils_1.toHex(transaction.nonce)
                            : undefined,
                    },
                ],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error signing transaction: ${error.message}`);
        }
    }
    /**
     * Submits a transaction object to the provider to be sign and sent to the network
     * @param {object} transaction Ethereum transaction
     * @param {string} transaction.from Address transaction will be sent from
     * @param {string} transaction.to Address transaction is directed towards (optional when creating new contract)
     * @param {string} transaction.gas Hex string representing gas to provide for transaction (ETH node defaults to 90,000)
     * @param {string} transaction.gasPrice Hex string representing price paid for each unit of gas in Wei (ETH node will determine if not provided)
     * @param {string} transaction.value Hex string representing number of Wei to send to {to}
     * @param {string} transaction.data Hex string representing compiled code of a contract or the hash of the invoked method signature and encoded parameters
     * @param {string} transaction.nonce Can be used to overwrite pending transactions that use the same nonce
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Transaction hash or zero hash if the transaction is not yet available
     */
    async sendTransaction(transaction, callOptions) {
        try {
            const requestParameters = [
                'eth_sendTransaction',
                [
                    {
                        ...transaction,
                        gas: transaction.gas
                            ? web3_utils_1.toHex(transaction.gas)
                            : undefined,
                        gasPrice: transaction.gasPrice
                            ? web3_utils_1.toHex(transaction.gasPrice)
                            : undefined,
                        value: transaction.value
                            ? web3_utils_1.toHex(transaction.value)
                            : undefined,
                        nonce: transaction.nonce
                            ? web3_utils_1.toHex(transaction.nonce)
                            : undefined,
                    },
                ],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error sending transaction: ${error.message}`);
        }
    }
    /**
     * Submits a previously signed transaction object to the network
     * @param {string} rawTransaction Hex string representing previously signed transaction
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Transaction hash or zero hash if the transaction is not yet available
     */
    async sendRawTransaction(rawTransaction, callOptions) {
        try {
            const requestParameters = ['eth_sendRawTransaction', [rawTransaction], callOptions];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error sending raw transaction: ${error.message}`);
        }
    }
    // TODO Discuss formatting result
    /**
     * Executes a new message call immediately without creating a transaction on the block chain
     * @param {object} transaction Ethereum transaction
     * @param {string} transaction.from Address transaction will be sent from
     * @param {string} transaction.to Address transaction is directed towards
     * @param {string} transaction.gas Hex string representing gas to provide for transaction (ETH node defaults to 90,000)
     * @param {string} transaction.gasPrice Hex string representing price paid for each unit of gas in Wei (ETH node will determine if not provided)
     * @param {string} transaction.value Hex string representing number of Wei to send to {to}
     * @param {string} transaction.data Hash of the method signature and encoded parameters
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing return value of executed contract
     */
    async call(transaction, callOptions) {
        try {
            const requestParameters = [
                'eth_call',
                [
                    {
                        ...transaction,
                        gas: transaction.gas
                            ? web3_utils_1.toHex(transaction.gas)
                            : undefined,
                        gasPrice: transaction.gasPrice
                            ? web3_utils_1.toHex(transaction.gasPrice)
                            : undefined,
                        value: transaction.value
                            ? web3_utils_1.toHex(transaction.value)
                            : undefined,
                    },
                ],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error sending call transaction: ${error.message}`);
        }
    }
    /**
     * Generates and returns an estimate of how much gas is necessary to allow the transaction to complete
     * @param {object} transaction Ethereum transaction
     * @param {string} transaction.from Address transaction will be sent from (optional)
     * @param {string} transaction.to Address transaction is directed towards (optional)
     * @param {string} transaction.gas Hex string representing gas to provide for transaction (ETH node defaults to 90,000)
     * @param {string} transaction.gasPrice Hex string representing price paid for each unit of gas in Wei (ETH node will determine if not provided)
     * @param {string} transaction.value Hex string representing number of Wei to send to {to} (optional)
     * @param {string} transaction.data Hash of the method signature and encoded parameters (optional)
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Hex string representing estimated amount of gas to be used
     */
    async estimateGas(transaction, callOptions) {
        try {
            const requestParameters = [
                'eth_estimateGas',
                [
                    {
                        ...transaction,
                        gas: transaction.gas
                            ? web3_utils_1.toHex(transaction.gas)
                            : undefined,
                        gasPrice: transaction.gasPrice
                            ? web3_utils_1.toHex(transaction.gasPrice)
                            : undefined,
                        value: transaction.value
                            ? web3_utils_1.toHex(transaction.value)
                            : undefined,
                        nonce: transaction.nonce
                            ? web3_utils_1.toHex(transaction.nonce)
                            : undefined,
                    },
                ],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting gas estimate: ${error.message}`);
        }
    }
    /**
     * Returns information about a block by hash
     * @param {string} blockHash Hash of block to get information for
     * @param {boolean} returnFullTxs If true it returns the full transaction objects, if false returns only the hashes of the transactions
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A block object or null when no block was found
     */
    async getBlockByHash(blockHash, returnFullTxs, callOptions) {
        try {
            const requestParameters = ['eth_getBlockByHash', [blockHash, returnFullTxs], callOptions];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, [
                    'number',
                    'difficulty',
                    'totalDifficulty',
                    'size',
                    'gasLimit',
                    'gasUsed',
                    'timestamp',
                    {
                        transactions: [
                            'blockNumber',
                            'gas',
                            'gasPrice',
                            'nonce',
                            'transactionIndex',
                            'value',
                            'v',
                        ],
                    },
                ], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting block by hash: ${error.message}`);
        }
    }
    /**
     * Returns information about a block by number
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {boolean} returnFullTxs If true it returns the full transaction objects, if false returns only the hashes of the transactions
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A block object or null when no block was found
     */
    async getBlockByNumber(blockIdentifier, returnFullTxs, callOptions) {
        try {
            const requestParameters = [
                'eth_getBlockByNumber',
                [web3_utils_1.toHex(blockIdentifier), returnFullTxs],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, [
                    'number',
                    'difficulty',
                    'totalDifficulty',
                    'size',
                    'gasLimit',
                    'gasUsed',
                    'timestamp',
                    {
                        transactions: [
                            'blockNumber',
                            'gas',
                            'gasPrice',
                            'nonce',
                            'transactionIndex',
                            'value',
                            'v',
                        ],
                    },
                ], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting block by number: ${error.message}`);
        }
    }
    /**
     * Returns the information about a transaction requested by transaction hash
     * @param {string} txHash Hash of transaction to retrieve
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A transaction object or {null} when no transaction was found
     */
    async getTransactionByHash(txHash, callOptions) {
        try {
            const requestParameters = ['eth_getTransactionByHash', [txHash], callOptions];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, [
                    'blockNumber',
                    'gas',
                    'gasPrice',
                    'nonce',
                    'transactionIndex',
                    'value',
                    'v',
                ], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting transaction by hash: ${error.message}`);
        }
    }
    /**
     * Returns information about a transaction by block hash and transaction index position
     * @param {string} blockHash Hash of block to get transactions of
     * @param {string} transactionIndex Hex string representing index of transaction to return
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A transaction object or {null} when no transaction was found
     */
    async getTransactionByBlockHashAndIndex(blockHash, transactionIndex, callOptions) {
        try {
            const requestParameters = [
                'eth_getTransactionByBlockHashAndIndex',
                [blockHash, web3_utils_1.toHex(transactionIndex)],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, [
                    'blockNumber',
                    'gas',
                    'gasPrice',
                    'nonce',
                    'transactionIndex',
                    'value',
                    'v',
                ], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting transaction by block hash and index: ${error.message}`);
        }
    }
    /**
     * Returns information about a transaction by block number and transaction index position
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {string} transactionIndex Hex string representing index of transaction to return
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A transaction object or {null} when no transaction was found
     */
    async getTransactionByBlockNumberAndIndex(blockIdentifier, transactionIndex, callOptions) {
        try {
            const requestParameters = [
                'eth_getTransactionByBlockNumberAndIndex',
                [web3_utils_1.toHex(blockIdentifier), web3_utils_1.toHex(transactionIndex)],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, [
                    'blockNumber',
                    'gas',
                    'gasPrice',
                    'nonce',
                    'transactionIndex',
                    'value',
                    'v',
                ], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting transaction by block number and index: ${error.message}`);
        }
    }
    /**
     * Returns the receipt of a transaction by transaction hash
     * @param {string} txHash Hash of transaction to get receipt of
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A transaction object or {null} when no receipt was found
     */
    async getTransactionReceipt(txHash, callOptions) {
        try {
            const requestParameters = ['eth_getTransactionReceipt', [txHash], callOptions];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, [
                    'transactionIndex',
                    'blockNumber',
                    'cumulativeGasUsed',
                    'gasUsed',
                    {
                        logs: [
                            'logIndex',
                            'transactionIndex',
                            'blockNumber',
                        ],
                    },
                    'status',
                ], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting transaction reciept: ${error.message}`);
        }
    }
    /**
     * Returns information about a uncle of a block by hash and uncle index position
     * @param {string} blockHash Hash of block to get uncles of
     * @param {string} uncleIndex Index of uncle to retrieve
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A block object or null when no block was found
     */
    async getUncleByBlockHashAndIndex(blockHash, uncleIndex, callOptions) {
        try {
            const requestParameters = [
                'eth_getUncleByBlockHashAndIndex',
                [blockHash, web3_utils_1.toHex(uncleIndex)],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, [
                    'number',
                    'difficulty',
                    'totalDifficulty',
                    'size',
                    'gasLimit',
                    'gasUsed',
                    'timestamp',
                    {
                        transactions: [
                            'blockNumber',
                            'gas',
                            'gasPrice',
                            'nonce',
                            'transactionIndex',
                            'value',
                            'v',
                        ],
                    },
                ], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting uncle by block hash and index: ${error.message}`);
        }
    }
    /**
     * Returns information about a uncle of a block by number and uncle index position
     * @param {string|number} blockIdentifier Integer or hex string representing block number, or "latest", "earliest", "pending"
     * @param {string} uncleIndex Index of uncle to retrieve
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A block object or null when no block was found
     */
    async getUncleByBlockNumberAndIndex(blockIdentifier, uncleIndex, callOptions) {
        try {
            const requestParameters = [
                'eth_getUncleByBlockNumberAndIndex',
                [web3_utils_1.toHex(blockIdentifier), web3_utils_1.toHex(uncleIndex)],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, [
                    'number',
                    'difficulty',
                    'totalDifficulty',
                    'size',
                    'gasLimit',
                    'gasUsed',
                    'timestamp',
                    {
                        transactions: [
                            'blockNumber',
                            'gas',
                            'gasPrice',
                            'nonce',
                            'transactionIndex',
                            'value',
                            'v',
                        ],
                    },
                ], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting uncle by block number and index: ${error.message}`);
        }
    }
    /**
     * Returns a list of available compilers in the client
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} A list of available compilers
     */
    async getCompilers(callOptions) {
        try {
            const requestParameters = [
                'eth_getCompilers',
                [],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting compilers: ${error.message}`);
        }
    }
    /**
     * Returns compiled solidity code
     * @param {string} sourceCode Solidity code to be compiled
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} compiled {sourceCode}
     */
    async compileSolidity(sourceCode, callOptions) {
        try {
            const requestParameters = ['eth_compileSolidity', [sourceCode], callOptions];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting compiling solidity code: ${error.message}`);
        }
    }
    /**
     * Returns compiled LLL code
     * @param {string} sourceCode LLL code to be compiled
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} compiled {sourceCode}
     */
    async compileLLL(sourceCode, callOptions) {
        try {
            const requestParameters = ['eth_compileLLL', [sourceCode], callOptions];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting compiling LLL code: ${error.message}`);
        }
    }
    /**
     * Returns compiled serpent code
     * @param {string} sourceCode Serpent code to be compiled
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} compiled {sourceCode}
     */
    async compileSerpent(sourceCode, callOptions) {
        try {
            const requestParameters = ['eth_compileSerpent', [sourceCode], callOptions];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting compiling serpent code: ${error.message}`);
        }
    }
    /**
     * Creates a filter object, based on filter options, to notify when the state changes (logs)
     * @param {object} filter Filter to be created
     * @param {string|number} filter.fromBlock Start filter at integer block number or "latest", "earliest", "pending"
     * @param {string|number} filter.toBlock End filter at integer block number or "latest", "earliest", "pending"
     * @param {string|string[]} filter.address: Contract address or list of addresses from which logs should originate
     * @param {string[]} filter.topics Topics to use for filtering (optional)
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Filter id
     */
    async newFilter(filter, callOptions) {
        try {
            const requestParameters = [
                'eth_newFilter',
                [
                    {
                        ...filter,
                        fromBlock: filter.fromBlock
                            ? web3_utils_1.toHex(filter.fromBlock)
                            : undefined,
                        toBlock: filter.toBlock
                            ? web3_utils_1.toHex(filter.toBlock)
                            : undefined,
                    },
                ],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error creating filter: ${error.message}`);
        }
    }
    /**
     * Creates a filter in the node, to notify when a new block arrives
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Filter id
     */
    async newBlockFilter(callOptions) {
        try {
            const requestParameters = [
                'eth_newBlockFilter',
                [],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error creating block filter: ${error.message}`);
        }
    }
    /**
     * Creates a filter in the node, to notify when new pending transactions arrive
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Filter id
     */
    async newPendingTransactionFilter(callOptions) {
        try {
            const requestParameters = [
                'eth_newPendingTransactionFilter',
                [],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutput(response.result, callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error creating pending transaction filter: ${error.message}`);
        }
    }
    /**
     * Uninstalls a filter with given id. Should always be called when watch is no longer needed
     * @param {string} filterId Id of filter to uninstall from node
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Returns true if filter was successfully uninstalled, otherwise false
     */
    async uninstallFilter(filterId, callOptions) {
        try {
            const requestParameters = ['eth_uninstallFilter', [web3_utils_1.toHex(filterId)], callOptions];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error uninstalling filter: ${error.message}`);
        }
    }
    // TODO Formatting output could be intensive since {response} is an array of results
    /**
     * Polling method for a filter, which returns an array of logs which occurred since last poll
     * @param {string} filterid Id of filter to retrieve changes from
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Array of log objects, or an empty array if nothing has changed since last poll
     */
    async getFilterChanges(filterId, callOptions) {
        try {
            const requestParameters = ['eth_getFilterChanges', [web3_utils_1.toHex(filterId)], callOptions];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, ['logIndex', 'transactionIndex', 'blockNumber'], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting filter changes: ${error.message}`);
        }
    }
    // TODO Formatting output could be intensive since {response} is an array of results
    /**
     * Returns an array of all logs matching filter with given id
     * @param {string} filterid Id of filter to retrieve
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Array of log objects, or an empty array if nothing has changed since last poll
     */
    async getFilterLogs(filterId, callOptions) {
        try {
            const requestParameters = ['eth_getFilterLogs', [web3_utils_1.toHex(filterId)], callOptions];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, ['logIndex', 'transactionIndex', 'blockNumber'], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting filter changes: ${error.message}`);
        }
    }
    // TODO Formatting output could be intensive since {response} is an array of results
    /**
     * Returns an array of all logs matching a given filter object
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Array of log objects, or an empty array if nothing has changed since last poll
     */
    async getLogs(filter, callOptions) {
        try {
            const requestParameters = [
                'eth_getLogs',
                [
                    {
                        ...filter,
                        fromBlock: filter.fromBlock
                            ? web3_utils_1.toHex(filter.fromBlock)
                            : undefined,
                        toBlock: filter.toBlock
                            ? web3_utils_1.toHex(filter.toBlock)
                            : undefined,
                    },
                ],
                callOptions,
            ];
            if (callOptions?.subscribe)
                return await this._subscribe(...requestParameters);
            const response = await this._send(...requestParameters);
            return {
                ...response,
                result: web3_utils_1.formatOutputObject(response.result, ['logIndex', 'transactionIndex', 'blockNumber'], callOptions?.returnType || this._defaultReturnType),
            };
        }
        catch (error) {
            throw Error(`Error getting logs: ${error.message}`);
        }
    }
    /**
     * Returns the hash of the current block, the seedHash, and the boundary condition to be met (“target”)
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Array of work info (in order: current block header pow-hash, seed hash used for the DAG, and boundary condition (“target”), 2^256 / difficulty)
     */
    async getWork(callOptions) {
        try {
            const requestParameters = [
                'eth_getWork',
                [],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error getting work: ${error.message}`);
        }
    }
    /**
     * Used for submitting a proof-of-work solution
     * @param {string} nonce Hex string representing found nonce (64 bits)
     * @param {string} powHash Hex string representing POW hash (256 bits)
     * @param {string} digest Hex string representing mix digest (256 bits)
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Returns true if the provided solution is valid, otherwise false
     */
    async submitWork(nonce, powHash, digest, callOptions) {
        try {
            const requestParameters = [
                'eth_submitWork',
                [web3_utils_1.toHex(nonce, 8), powHash, digest],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error submitting work: ${error.message}`);
        }
    }
    /**
     * Used for submitting mining hashrate
     * @param {string} hashRate Hex string representing desired hash rate (32 bytes)
     * @param {string} clientId Hex string representing ID identifying the client
     * @param {object} callOptions
     * @param {object} callOptions.providerCallOptions Varies based on provider type (e.g. HttpOptions)
     * @param {object} callOptions.rpcOptions
     * @param {number} callOptions.rpcOptions.id (Optional) Id to pass along with RPC request. Gets returned by RPC node
     * @param {string} callOptions.rpcOptions.jsonrpc (Optional) Version of JSON RPC version
     * @param {string} callOptions.returnType (Optional) Desired return type of formattable outputs
     * @param {boolean} callOptions.subscribe (Optional) If true, returns Node.js EventEmitter and polls
     * @returns {Promise} Returns true if the provided solution is valid, otherwise false
     */
    async submitHashRate(hashRate, clientId, callOptions) {
        try {
            const requestParameters = [
                'eth_submitHashRate',
                [web3_utils_1.toHex(hashRate, 32), web3_utils_1.toHex(clientId)],
                callOptions,
            ];
            return callOptions?.subscribe
                ? await this._subscribe(...requestParameters)
                : await this._send(...requestParameters);
        }
        catch (error) {
            throw Error(`Error submitting hash rate: ${error.message}`);
        }
    }
}
exports.default = Web3Eth;
